@@ -1,25 +1,24 @@
 import copy as _copy
-import numpy as np
 import operator
-from numpy.lib.mixins import NDArrayOperatorsMixin
-from functools import reduce
 from collections.abc import Iterable
+from functools import reduce
+
+import numpy as np
 import scipy.sparse as ss
-from scipy.sparse import compressed
-from typing import Tuple
+from numpy.lib.mixins import NDArrayOperatorsMixin
 
-from .._sparse_array import SparseArray, _reduce_super_ufunc
 from .._coo.common import linear_loc
-from .._common import dot, matmul
-from .._utils import (
-    normalize_axis,
-    can_store,
-    check_zero_fill_value,
-    check_compressed_axes,
-    equivalent,
-)
 from .._coo.core import COO
-from .convert import uncompress_dimension, _transpose, _1d_reshape
+from .._sparse_array import SparseArray
+from .._utils import (
+    _zero_of_dtype,
+    can_store,
+    check_compressed_axes,
+    check_fill_value,
+    equivalent,
+    normalize_axis,
+)
+from .convert import _1d_reshape, _transpose, uncompress_dimension
 from .indexing import getitem
 
 
@@ -53,11 +52,7 @@ def _from_coo(x, compressed_axes=None, idx_dtype=None):
 
     if idx_dtype and not can_store(idx_dtype, max(max(compressed_shape), x.nnz)):
         raise ValueError(
-            "cannot store array with the compressed shape {} and nnz {} with dtype {}.".format(
-                compressed_shape,
-                x.nnz,
-                idx_dtype,
-            )
+            f"cannot store array with the compressed shape {compressed_shape} and nnz {x.nnz} with dtype {idx_dtype}."
         )
 
     if not idx_dtype:
@@ -143,21 +138,17 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         shape=None,
         compressed_axes=None,
         prune=False,
-        fill_value=0,
+        fill_value=None,
         idx_dtype=None,
     ):
         if isinstance(arg, ss.spmatrix):
             arg = self.from_scipy_sparse(arg)
 
         if isinstance(arg, np.ndarray):
-            (arg, shape, compressed_axes, fill_value) = _from_coo(
-                COO(arg), compressed_axes
-            )
+            (arg, shape, compressed_axes, fill_value) = _from_coo(COO(arg), compressed_axes)
 
         elif isinstance(arg, COO):
-            (arg, shape, compressed_axes, fill_value) = _from_coo(
-                arg, compressed_axes, idx_dtype
-            )
+            (arg, shape, compressed_axes, fill_value) = _from_coo(arg, compressed_axes, idx_dtype)
 
         elif isinstance(arg, GCXS):
             if compressed_axes is not None and arg.compressed_axes != compressed_axes:
@@ -184,10 +175,11 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
 
         self.shape = shape
 
-        self._compressed_axes = (
-            tuple(compressed_axes) if isinstance(compressed_axes, Iterable) else None
-        )
-        self.fill_value = fill_value
+        if fill_value is None:
+            fill_value = _zero_of_dtype(self.data.dtype)
+
+        self._compressed_axes = tuple(compressed_axes) if isinstance(compressed_axes, Iterable) else None
+        self.fill_value = self.data.dtype.type(fill_value)
 
         if prune:
             self._prune()
@@ -204,35 +196,25 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         return _copy.deepcopy(self) if deep else _copy.copy(self)
 
     @classmethod
-    def from_numpy(cls, x, compressed_axes=None, fill_value=0, idx_dtype=None):
-        coo = COO(x, fill_value=fill_value, idx_dtype=idx_dtype)
+    def from_numpy(cls, x, compressed_axes=None, fill_value=None, idx_dtype=None):
+        coo = COO.from_numpy(x, fill_value=fill_value, idx_dtype=idx_dtype)
         return cls.from_coo(coo, compressed_axes, idx_dtype)
 
     @classmethod
     def from_coo(cls, x, compressed_axes=None, idx_dtype=None):
-        (arg, shape, compressed_axes, fill_value) = _from_coo(
-            x, compressed_axes, idx_dtype
-        )
-        return cls(
-            arg, shape=shape, compressed_axes=compressed_axes, fill_value=fill_value
-        )
+        (arg, shape, compressed_axes, fill_value) = _from_coo(x, compressed_axes, idx_dtype)
+        return cls(arg, shape=shape, compressed_axes=compressed_axes, fill_value=fill_value)
 
     @classmethod
-    def from_scipy_sparse(cls, x):
+    def from_scipy_sparse(cls, x, /, *, fill_value=None):
         if x.format == "csc":
-            return cls(
-                (x.data, x.indices, x.indptr), shape=x.shape, compressed_axes=(1,)
-            )
-        else:
-            x = x.asformat("csr")
-            return cls(
-                (x.data, x.indices, x.indptr), shape=x.shape, compressed_axes=(0,)
-            )
+            return cls((x.data, x.indices, x.indptr), shape=x.shape, compressed_axes=(1,), fill_value=fill_value)
+
+        x = x.asformat("csr")
+        return cls((x.data, x.indices, x.indptr), shape=x.shape, compressed_axes=(0,), fill_value=fill_value)
 
     @classmethod
-    def from_iter(
-        cls, x, shape=None, compressed_axes=None, fill_value=None, idx_dtype=None
-    ):
+    def from_iter(cls, x, shape=None, compressed_axes=None, fill_value=None, idx_dtype=None):
         return cls.from_coo(
             COO.from_iter(x, shape, fill_value),
             compressed_axes,
@@ -289,10 +271,10 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         Examples
         -------
         >>> import sparse
-        >>> s = sparse.random((5,5), density=0.2, format='dok')
+        >>> s = sparse.random((5, 5), density=0.2, format="dok")
         >>> s.format
         'dok'
-        >>> t = sparse.random((5,5), density=0.2, format='coo')
+        >>> t = sparse.random((5, 5), density=0.2, format="coo")
         >>> t.format
         'coo'
         """
@@ -313,15 +295,12 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         --------
         numpy.ndarray.nbytes : The equivalent Numpy property.
         """
-        nbytes = self.data.nbytes + self.indices.nbytes + self.indptr.nbytes
-        return nbytes
+        return self.data.nbytes + self.indices.nbytes + self.indptr.nbytes
 
     @property
     def _axis_order(self):
         axis_order = list(self.compressed_axes)
-        axis_order.extend(
-            np.setdiff1d(np.arange(len(self.shape)), self.compressed_axes)
-        )
+        axis_order.extend(np.setdiff1d(np.arange(len(self.shape)), self.compressed_axes))
         return axis_order
 
     @property
@@ -344,8 +323,9 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         return self.transpose()
 
     def __str__(self):
-        return "<GCXS: shape={}, dtype={}, nnz={}, fill_value={}, compressed_axes={}>".format(
-            self.shape, self.dtype, self.nnz, self.fill_value, self.compressed_axes
+        return self._str_impl(
+            f"<GCXS: shape={self.shape}, dtype={self.dtype}, nnz={self.nnz}, "
+            f"fill_value={self.fill_value}, compressed_axes={self.compressed_axes}>"
         )
 
     __repr__ = __str__
@@ -403,8 +383,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
             raise NotImplementedError("no axes to compress for 1d array")
 
         new_compressed_axes = tuple(
-            normalize_axis(new_compressed_axes[i], self.ndim)
-            for i in range(len(new_compressed_axes))
+            normalize_axis(new_compressed_axes[i], self.ndim) for i in range(len(new_compressed_axes))
         )
 
         if new_compressed_axes == self.compressed_axes:
@@ -493,13 +472,20 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
 
         return DOK.from_coo(self.tocoo())  # probably a temporary solution
 
-    def to_scipy_sparse(self):
+    def to_scipy_sparse(self, accept_fv=None):
         """
         Converts this :obj:`GCXS` object into a :obj:`scipy.sparse.csr_matrix` or `scipy.sparse.csc_matrix`.
+
+        Parameters
+        ----------
+        accept_fv : scalar or list of scalar, optional
+            The list of accepted fill-values. The default accepts only zero.
+
         Returns
         -------
         :obj:`scipy.sparse.csr_matrix` or `scipy.sparse.csc_matrix`
             The converted Scipy sparse matrix.
+
         Raises
         ------
         ValueError
@@ -508,21 +494,14 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
             If all the array doesn't zero fill-values.
         """
 
-        check_zero_fill_value(self)
-
+        check_fill_value(self, accept_fv=accept_fv)
         if self.ndim != 2:
-            raise ValueError(
-                "Can only convert a 2-dimensional array to a Scipy sparse matrix."
-            )
+            raise ValueError("Can only convert a 2-dimensional array to a Scipy sparse matrix.")
 
         if 0 in self.compressed_axes:
-            return ss.csr_matrix(
-                (self.data, self.indices, self.indptr), shape=self.shape
-            )
-        else:
-            return ss.csc_matrix(
-                (self.data, self.indices, self.indptr), shape=self.shape
-            )
+            return ss.csr_matrix((self.data, self.indices, self.indptr), shape=self.shape)
+
+        return ss.csc_matrix((self.data, self.indices, self.indptr), shape=self.shape)
 
     def asformat(self, format, **kwargs):
         """
@@ -591,12 +570,10 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
             If the returned array would be too large.
         """
 
-        if self.size <= max_size or self.density >= min_density:
-            return self.todense()
-        else:
-            raise ValueError(
-                "Operation would require converting " "large sparse array to dense"
-            )
+        if self.size > max_size and self.density < min_density:
+            raise ValueError("Operation would require converting large sparse array to dense")
+
+        return self.todense()
 
     def flatten(self, order="C"):
         """
@@ -613,7 +590,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         Numpy and isn't actually supported.
         """
         if order not in {"C", None}:
-            raise NotImplementedError("The `order` parameter is not" "supported.")
+            raise NotImplementedError("The `order` parameter is not supported.")
 
         return self.reshape(-1)
 
@@ -645,10 +622,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         Numpy and isn't actually supported.
 
         """
-        if isinstance(shape, Iterable):
-            shape = tuple(shape)
-        else:
-            shape = (shape,)
+        shape = tuple(shape) if isinstance(shape, Iterable) else (shape,)
         if order not in {"C", None}:
             raise NotImplementedError("The 'order' parameter is not supported")
         if any(d == -1 for d in shape):
@@ -659,9 +633,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
             return self
 
         if self.size != reduce(operator.mul, shape, 1):
-            raise ValueError(
-                "cannot reshape array of size {} into shape {}".format(self.size, shape)
-            )
+            raise ValueError(f"cannot reshape array of size {self.size} into shape {shape}")
         if len(shape) == 0:
             return self.tocoo().reshape(shape).asformat("gcxs")
 
@@ -759,11 +731,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         numpy.ndarray.transpose : Numpy equivalent function.
         """
         if self.ndim != 2:
-            raise ValueError(
-                "cannot perform 2d transpose on array with dimension {}".format(
-                    self.ndim
-                )
-            )
+            raise ValueError(f"cannot perform 2d transpose on array with dimension {self.ndim}")
 
         compressed_axes = [(self.compressed_axes[0] + 1) % 2]
         shape = self.shape[::-1]
@@ -800,15 +768,21 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         :obj:`numpy.dot` : Numpy equivalent function.
         scipy.sparse.csr_matrix.dot : Scipy equivalent function.
         """
+        from .._common import dot
+
         return dot(self, other)
 
     def __matmul__(self, other):
+        from .._common import matmul
+
         try:
             return matmul(self, other)
         except NotImplementedError:
             return NotImplemented
 
     def __rmatmul__(self, other):
+        from .._common import matmul
+
         try:
             return matmul(other, self)
         except NotImplementedError:
@@ -823,7 +797,7 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         --------
         >>> coords = np.array([[0, 1, 2, 3]])
         >>> data = np.array([1, 0, 1, 2])
-        >>> s = COO(coords, data).asformat('gcxs')
+        >>> s = COO(coords, data).asformat("gcxs")
         >>> s._prune()
         >>> s.nnz
         3
@@ -842,11 +816,15 @@ class GCXS(SparseArray, NDArrayOperatorsMixin):
         else:
             self.indices = self.indices[mask]
 
+    def isinf(self):
+        return self.tocoo().isinf().asformat("gcxs", compressed_axes=self.compressed_axes)
+
+    def isnan(self):
+        return self.tocoo().isnan().asformat("gcxs", compressed_axes=self.compressed_axes)
+
 
 class _Compressed2d(GCXS):
-    def __init__(
-        self, arg, shape=None, compressed_axes=None, prune=False, fill_value=0
-    ):
+    def __init__(self, arg, shape=None, compressed_axes=None, prune=False, fill_value=0):
         if not hasattr(arg, "shape") and shape is None:
             raise ValueError("missing `shape` argument")
         if shape is not None and hasattr(arg, "shape"):
@@ -864,12 +842,9 @@ class _Compressed2d(GCXS):
         )
 
     def __str__(self):
-        return "<{}: shape={}, dtype={}, nnz={}, fill_value={}>".format(
-            type(self).__name__,
-            self.shape,
-            self.dtype,
-            self.nnz,
-            self.fill_value,
+        return self._str_impl(
+            f"<{type(self).__name__}: shape={self.shape}, dtype={self.dtype}, "
+            f"nnz={self.nnz}, fill_value={self.fill_value}>"
         )
 
     __repr__ = __str__
@@ -892,13 +867,13 @@ class CSR(_Compressed2d):
         super().__init__(arg, shape=shape, compressed_axes=(0,), fill_value=fill_value)
 
     @classmethod
-    def from_scipy_sparse(cls, x):
+    def from_scipy_sparse(cls, x, /, *, fill_value=None):
         x = x.asformat("csr", copy=False)
-        return cls((x.data, x.indices, x.indptr), shape=x.shape)
+        return cls((x.data, x.indices, x.indptr), shape=x.shape, fill_value=fill_value)
 
     def transpose(self, axes: None = None, copy: bool = False) -> "CSC":
         if axes is not None:
-            raise ValueError()
+            raise ValueError
         if copy:
             self = self.copy()
         return CSC((self.data, self.indices, self.indptr), self.shape[::-1])
@@ -917,13 +892,13 @@ class CSC(_Compressed2d):
         super().__init__(arg, shape=shape, compressed_axes=(1,), fill_value=fill_value)
 
     @classmethod
-    def from_scipy_sparse(cls, x):
+    def from_scipy_sparse(cls, x, /, *, fill_value=None):
         x = x.asformat("csc", copy=False)
-        return cls((x.data, x.indices, x.indptr), shape=x.shape)
+        return cls((x.data, x.indices, x.indptr), shape=x.shape, fill_value=fill_value)
 
     def transpose(self, axes: None = None, copy: bool = False) -> CSR:
         if axes is not None:
-            raise ValueError()
+            raise ValueError
         if copy:
             self = self.copy()
         return CSR((self.data, self.indices, self.indptr), self.shape[::-1])
