@@ -1,26 +1,27 @@
 import copy as _copy
 import operator
-from collections.abc import Iterable, Iterator, Sized
-from collections import defaultdict, deque
-from functools import reduce
 import warnings
+from collections import defaultdict, deque
+from collections.abc import Iterable, Iterator, Sized
+from functools import reduce
+
+import numba
 
 import numpy as np
-import numba
 import scipy.sparse
 from numpy.lib.mixins import NDArrayOperatorsMixin
 
-from .._common import dot, matmul
-from .indexing import getitem
-from .._umath import elemwise, broadcast_to
-from .._sparse_array import SparseArray, _reduce_super_ufunc
+from .._sparse_array import SparseArray
+from .._umath import broadcast_to
 from .._utils import (
-    normalize_axis,
-    equivalent,
-    check_zero_fill_value,
     _zero_of_dtype,
     can_store,
+    check_fill_value,
+    check_zero_fill_value,
+    equivalent,
+    normalize_axis,
 )
+from .indexing import getitem
 
 
 class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
@@ -115,14 +116,14 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
     :obj:`COO` objects also support dot products and reductions.
 
-    >>> s.dot(s.T).sum(axis=0).todense()   # doctest: +NORMALIZE_WHITESPACE
+    >>> s.dot(s.T).sum(axis=0).todense()  # doctest: +NORMALIZE_WHITESPACE
     array([ 1,  1, 31,  6], dtype=uint64)
 
     You can use Numpy :code:`ufunc` operations on :obj:`COO` arrays as well.
 
     >>> np.sum(s, axis=1).todense()  # doctest: +NORMALIZE_WHITESPACE
     array([1, 1, 6, 1], dtype=uint64)
-    >>> np.round(np.sqrt(s, dtype=np.float64), decimals=1).todense()   # doctest: +SKIP
+    >>> np.round(np.sqrt(s, dtype=np.float64), decimals=1).todense()  # doctest: +SKIP
     array([[ 1. ,  0. ,  0. ,  0. ],
            [ 0. ,  1. ,  0. ,  0. ],
            [ 0. ,  0. ,  1. ,  2.2],
@@ -136,9 +137,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
     You can also create :obj:`COO` arrays from coordinates and data.
 
-    >>> coords = [[0, 0, 0, 1, 1],
-    ...           [0, 1, 2, 0, 3],
-    ...           [0, 3, 2, 0, 1]]
+    >>> coords = [[0, 0, 0, 1, 1], [0, 1, 2, 0, 3], [0, 3, 2, 0, 1]]
     >>> data = [1, 2, 3, 4, 5]
     >>> s4 = COO(coords, data, shape=(3, 4, 5))
     >>> s4
@@ -146,9 +145,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
     If the data is same across all coordinates, you can also specify a scalar.
 
-    >>> coords = [[0, 0, 0, 1, 1],
-    ...           [0, 1, 2, 0, 3],
-    ...           [0, 3, 2, 0, 1]]
+    >>> coords = [[0, 0, 0, 1, 1], [0, 1, 2, 0, 3], [0, 3, 2, 0, 1]]
     >>> data = 1
     >>> s5 = COO(coords, data, shape=(3, 4, 5))
     >>> s5
@@ -174,9 +171,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
     >>> d = {(0, 0, 0): 1, (1, 2, 3): 2, (1, 1, 0): 3}
     >>> COO(d)
     <COO: shape=(2, 3, 4), dtype=int64, nnz=3, fill_value=0>
-    >>> L = [((0, 0), 1),
-    ...      ((1, 1), 2),
-    ...      ((0, 0), 3)]
+    >>> L = [((0, 0), 1), ((1, 1), 2), ((0, 0), 3)]
     >>> COO(L).todense()  # doctest: +NORMALIZE_WHITESPACE
     array([[4, 0],
            [0, 2]])
@@ -188,7 +183,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
     >>> s6[1:3, 1:3] = [[4, 5], [6, 7]]
     >>> s6
     <DOK: shape=(5, 5), dtype=int64, nnz=4, fill_value=0>
-    >>> s7 = s6.asformat('coo')
+    >>> s7 = s6.asformat("coo")
     >>> s7
     <COO: shape=(5, 5), dtype=int64, nnz=4, fill_value=0>
     >>> s7.todense()  # doctest: +NORMALIZE_WHITESPACE
@@ -217,10 +212,15 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         if cache:
             self.enable_caching()
 
-        if data is None:
-            arr = as_coo(
-                coords, shape=shape, fill_value=fill_value, idx_dtype=idx_dtype
+        if not isinstance(coords, np.ndarray):
+            warnings.warn(
+                "coords should be an ndarray. This will raise a ValueError in the future.",
+                DeprecationWarning,
+                stacklevel=1,
             )
+
+        if data is None:
+            arr = as_coo(coords, shape=shape, fill_value=fill_value, idx_dtype=idx_dtype)
             self._make_shallow_copy_of(arr)
             if cache:
                 self.enable_caching()
@@ -245,11 +245,9 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             warnings.warn(
                 "shape should be provided. This will raise a ValueError in the future.",
                 DeprecationWarning,
+                stacklevel=1,
             )
-            if self.coords.nbytes:
-                shape = tuple((self.coords.max(axis=1) + 1))
-            else:
-                shape = ()
+            shape = tuple(self.coords.max(axis=1) + 1) if self.coords.nbytes else ()
 
         if not isinstance(shape, Iterable):
             shape = (shape,)
@@ -258,30 +256,17 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             shape = tuple(shape)
 
         if shape and not self.coords.size:
-            warnings.warn(
-                "coords should be an ndarray. This will raise a ValueError in the future.",
-                DeprecationWarning,
-            )
-            self.coords = np.zeros(
-                (len(shape) if isinstance(shape, Iterable) else 1, 0), dtype=np.intp
-            )
+            self.coords = np.zeros((len(shape) if isinstance(shape, Iterable) else 1, 0), dtype=np.intp)
 
         super().__init__(shape, fill_value=fill_value)
         if idx_dtype:
             if not can_store(idx_dtype, max(shape)):
-                raise ValueError(
-                    "cannot cast array with shape {} to dtype {}.".format(
-                        shape, idx_dtype
-                    )
-                )
+                raise ValueError(f"cannot cast array with shape {shape} to dtype {idx_dtype}.")
             self.coords = self.coords.astype(idx_dtype)
 
         if self.shape:
             if len(self.data) != self.coords.shape[1]:
-                msg = (
-                    "The data length does not match the coordinates "
-                    "given.\nlen(data) = {}, but {} coords specified."
-                )
+                msg = "The data length does not match the coordinates given.\nlen(data) = {}, but {} coords specified."
                 raise ValueError(msg.format(len(data), self.coords.shape[1]))
             if len(self.shape) != self.coords.shape[0]:
                 msg = (
@@ -289,9 +274,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
                     "shape of `coords`; len(shape)={} != coords.shape[0]={}"
                     "(and coords.shape={})"
                 )
-                raise ValueError(
-                    msg.format(len(shape), self.coords.shape[0], self.coords.shape)
-                )
+                raise ValueError(msg.format(len(shape), self.coords.shape[0], self.coords.shape))
 
         from .._settings import WARN_ON_TOO_DENSE
 
@@ -301,6 +284,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
                 "memory than than an equivalent dense array. You may want to "
                 "use a dense array here instead.",
                 RuntimeWarning,
+                stacklevel=1,
             )
 
         if not sorted:
@@ -323,9 +307,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         "Produce a deterministic, content-based hash for dask."
         from dask.base import normalize_token
 
-        return normalize_token(
-            (type(self), self.coords, self.data, self.shape, self.fill_value)
-        )
+        return normalize_token((type(self), self.coords, self.data, self.shape, self.fill_value))
 
     def copy(self, deep=True):
         """Return a copy of the array.
@@ -445,7 +427,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         return x
 
     @classmethod
-    def from_scipy_sparse(cls, x):
+    def from_scipy_sparse(cls, x, /, *, fill_value=None):
         """
         Construct a :obj:`COO` array from a :obj:`scipy.sparse.spmatrix`
 
@@ -453,6 +435,8 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         ----------
         x : scipy.sparse.spmatrix
             The sparse matrix to construct the array from.
+        fill_value : scalar
+            The fill-value to use when converting.
 
         Returns
         -------
@@ -476,6 +460,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             shape=x.shape,
             has_duplicates=not x.has_canonical_format,
             sorted=x.has_canonical_format,
+            fill_value=fill_value,
         )
 
     @classmethod
@@ -558,10 +543,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             data = np.array([item[1] for item in x], dtype=dtype)
 
         if not (
-            coords.ndim == 2
-            and data.ndim == 1
-            and np.issubdtype(coords.dtype, np.integer)
-            and np.all(coords >= 0)
+            coords.ndim == 2 and data.ndim == 1 and np.issubdtype(coords.dtype, np.integer) and np.all(coords >= 0)
         ):
             raise ValueError("Invalid iterable to convert to COO.")
 
@@ -637,10 +619,10 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         Examples
         -------
         >>> import sparse
-        >>> s = sparse.random((5,5), density=0.2, format='dok')
+        >>> s = sparse.random((5, 5), density=0.2, format="dok")
         >>> s.format
         'dok'
-        >>> t = sparse.random((5,5), density=0.2, format='coo')
+        >>> t = sparse.random((5, 5), density=0.2, format="coo")
         >>> t.format
         'coo'
         """
@@ -700,9 +682,8 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
     __getitem__ = getitem
 
     def __str__(self):
-        return "<COO: shape={!s}, dtype={!s}, nnz={:d}, fill_value={!s}>".format(
-            self.shape, self.dtype, self.nnz, self.fill_value
-        )
+        summary = f"<COO: shape={self.shape!s}, dtype={self.dtype!s}, nnz={self.nnz:d}, fill_value={self.fill_value!s}>"
+        return self._str_impl(summary)
 
     __repr__ = __str__
 
@@ -891,9 +872,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         <COO: shape=(4, 3, 2), dtype=float64, nnz=24, fill_value=0.0>
         """
         # Normalize all axis1, axis2 to positive values
-        axis1, axis2 = normalize_axis(
-            (axis1, axis2), self.ndim
-        )  # checks if axis1,2 are in range + raises ValueError
+        axis1, axis2 = normalize_axis((axis1, axis2), self.ndim)  # checks if axis1,2 are in range + raises ValueError
         axes = list(range(self.ndim))
         axes[axis1], axes[axis2] = axes[axis2], axes[axis1]
         return self.transpose(axes)
@@ -928,19 +907,25 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         --------
         >>> x = np.arange(4).reshape((2, 2))
         >>> s = COO.from_numpy(x)
-        >>> s.dot(s) # doctest: +SKIP
+        >>> s.dot(s)  # doctest: +SKIP
         array([[ 2,  3],
                [ 6, 11]], dtype=int64)
         """
+        from .._common import dot
+
         return dot(self, other)
 
     def __matmul__(self, other):
+        from .._common import matmul
+
         try:
             return matmul(self, other)
         except NotImplementedError:
             return NotImplemented
 
     def __rmatmul__(self, other):
+        from .._common import matmul
+
         try:
             return matmul(other, self)
         except NotImplementedError:
@@ -995,7 +980,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
         """
         if order not in {"C", None}:
-            raise NotImplementedError("The `order` parameter is not" "supported.")
+            raise NotImplementedError("The `order` parameter is notsupported.")
 
         return self.reshape(-1)
 
@@ -1033,10 +1018,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
                [15, 16, 17, 18, 19],
                [20, 21, 22, 23, 24]])
         """
-        if isinstance(shape, Iterable):
-            shape = tuple(shape)
-        else:
-            shape = (shape,)
+        shape = tuple(shape) if isinstance(shape, Iterable) else (shape,)
 
         if order not in {"C", None}:
             raise NotImplementedError("The `order` parameter is not supported")
@@ -1048,9 +1030,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             shape = tuple([d if d != -1 else extra for d in shape])
 
         if self.size != reduce(operator.mul, shape, 1):
-            raise ValueError(
-                "cannot reshape array of size {} into shape {}".format(self.size, shape)
-            )
+            raise ValueError(f"cannot reshape array of size {self.size} into shape {shape}")
 
         if self._cache is not None:
             for sh, value in self._cache["reshape"]:
@@ -1083,6 +1063,57 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             self._cache["reshape"].append((shape, result))
         return result
 
+    def squeeze(self, axis=None):
+        """
+        Removes singleton dimensions (axes) from ``x``.
+        Parameters
+        ----------
+        axis : Union[None, int, Tuple[int, ...]]
+            The axis (or axes) to squeeze. If a specified axis has a size greater than one,
+            a `ValueError` is raised. ``axis=None`` removes all singleton dimensions.
+            Default: ``None``.
+        Returns
+        -------
+        COO
+            The output array without ``axis`` dimensions.
+        Examples
+        --------
+        >>> s = COO.from_numpy(np.eye(2)).reshape((2, 1, 2, 1))
+        >>> s.squeeze().shape
+        (2, 2)
+        >>> s.squeeze(axis=1).shape
+        (2, 2, 1)
+        """
+        squeezable_dims = tuple([d for d in range(self.ndim) if self.shape[d] == 1])
+
+        if axis is None:
+            axis = squeezable_dims
+        if isinstance(axis, int):
+            axis = (axis,)
+        elif isinstance(axis, Iterable):
+            axis = tuple(axis)
+        else:
+            raise ValueError(f"Invalid axis parameter: `{axis}`.")
+
+        for d in axis:
+            if d not in squeezable_dims:
+                raise ValueError(f"Specified axis `{d}` has a size greater than one: {self.shape[d]}")
+
+        retained_dims = [d for d in range(self.ndim) if d not in axis]
+
+        coords = self.coords[retained_dims, :]
+        shape = tuple([s for idx, s in enumerate(self.shape) if idx in retained_dims])
+
+        return COO(
+            coords,
+            self.data,
+            shape,
+            has_duplicates=False,
+            sorted=True,
+            cache=self._cache is not None,
+            fill_value=self.fill_value,
+        )
+
     def resize(self, *args, refcheck=True, coords_dtype=np.intp):
         """
         This method changes the shape and size of an array in-place.
@@ -1096,9 +1127,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         numpy.ndarray.resize : The equivalent Numpy function.
 
         """
-        warnings.warn(
-            "resize is deprecated on all SpraseArray objects.", DeprecationWarning
-        )
+        warnings.warn("resize is deprecated on all SpraseArray objects.", DeprecationWarning, stacklevel=1)
         if len(args) == 1 and isinstance(args[0], tuple):
             shape = args[0]
         elif all(isinstance(arg, int) for arg in args):
@@ -1131,10 +1160,15 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         if len(self.data) != len(linear_loc):
             self.data = self.data[:end_idx].copy()
 
-    def to_scipy_sparse(self):
+    def to_scipy_sparse(self, /, *, accept_fv=None):
         """
         Converts this :obj:`COO` object into a :obj:`scipy.sparse.coo_matrix`.
 
+        Parameters
+        ----------
+        accept_fv : scalar or list of scalar, optional
+            The list of accepted fill-values. The default accepts only zero.
+
         Returns
         -------
         :obj:`scipy.sparse.coo_matrix`
@@ -1152,24 +1186,20 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         COO.tocsr : Convert to a :obj:`scipy.sparse.csr_matrix`.
         COO.tocsc : Convert to a :obj:`scipy.sparse.csc_matrix`.
         """
-        check_zero_fill_value(self)
+        import scipy.sparse
+
+        check_fill_value(self, accept_fv=accept_fv)
 
         if self.ndim != 2:
-            raise ValueError(
-                "Can only convert a 2-dimensional array to a Scipy sparse matrix."
-            )
+            raise ValueError("Can only convert a 2-dimensional array to a Scipy sparse matrix.")
 
-        result = scipy.sparse.coo_matrix(
-            (self.data, (self.coords[0], self.coords[1])), shape=self.shape
-        )
+        result = scipy.sparse.coo_matrix((self.data, (self.coords[0], self.coords[1])), shape=self.shape)
         result.has_canonical_format = True
         return result
 
     def _tocsr(self):
         if self.ndim != 2:
-            raise ValueError(
-                "This array must be two-dimensional for this conversion " "to work."
-            )
+            raise ValueError("This array must be two-dimensional for this conversion to work.")
         row, col = self.coords
 
         # Pass 3: count nonzeros in each row
@@ -1304,7 +1334,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         array([6, 7, 2], dtype=uint8)
         """
         # Inspired by scipy/sparse/coo.py::sum_duplicates
-        # See https://github.com/scipy/scipy/blob/master/LICENSE.txt
+        # See https://github.com/scipy/scipy/blob/main/LICENSE.txt
         linear = self.linear_loc()
         unique_mask = np.diff(linear) != 0
 
@@ -1407,12 +1437,10 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             ...
         ValueError: Operation would require converting large sparse array to dense
         """
-        if self.size <= max_size or self.density >= min_density:
-            return self.todense()
-        else:
-            raise ValueError(
-                "Operation would require converting " "large sparse array to dense"
-            )
+        if self.size > max_size and self.density < min_density:
+            raise ValueError("Operation would require converting large sparse array to dense")
+
+        return self.todense()
 
     def nonzero(self):
         """
@@ -1469,7 +1497,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
             return GCXS.from_coo(self, **kwargs)
 
-        elif len(kwargs) != 0:
+        if len(kwargs) != 0:
             raise TypeError(f"Invalid keyword arguments provided: {kwargs}")
 
         if format == "coo":
@@ -1482,6 +1510,36 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
         return self.asformat("gcxs", **kwargs).asformat(format, **kwargs)
 
+    def isinf(self):
+        """
+        Tests each element ``x_i`` of the array to determine if equal to positive or negative infinity.
+        """
+        new_fill_value = bool(np.isinf(self.fill_value))
+        new_data = np.isinf(self.data)
+
+        return COO(
+            self.coords,
+            new_data,
+            shape=self.shape,
+            fill_value=new_fill_value,
+            prune=True,
+        )
+
+    def isnan(self):
+        """
+        Tests each element ``x_i`` of the array to determine whether the element is ``NaN``.
+        """
+        new_fill_value = bool(np.isnan(self.fill_value))
+        new_data = np.isnan(self.data)
+
+        return COO(
+            self.coords,
+            new_data,
+            shape=self.shape,
+            fill_value=new_fill_value,
+            prune=True,
+        )
+
 
 def as_coo(x, shape=None, fill_value=None, idx_dtype=None):
     """
@@ -1511,21 +1569,15 @@ def as_coo(x, shape=None, fill_value=None, idx_dtype=None):
         Convert an iterable to :obj:`COO`.
     """
     if hasattr(x, "shape") and shape is not None:
-        raise ValueError(
-            "Cannot provide a shape in combination with something "
-            "that already has a shape."
-        )
+        raise ValueError("Cannot provide a shape in combination with something that already has a shape.")
 
     if hasattr(x, "fill_value") and fill_value is not None:
-        raise ValueError(
-            "Cannot provide a fill-value in combination with something "
-            "that already has a fill-value."
-        )
+        raise ValueError("Cannot provide a fill-value in combination with something that already has a fill-value.")
 
     if isinstance(x, SparseArray):
         return x.asformat("coo")
 
-    if isinstance(x, np.ndarray):
+    if isinstance(x, np.ndarray) or np.isscalar(x):
         return COO.from_numpy(x, fill_value=fill_value, idx_dtype=idx_dtype)
 
     if isinstance(x, scipy.sparse.spmatrix):
@@ -1535,8 +1587,8 @@ def as_coo(x, shape=None, fill_value=None, idx_dtype=None):
         return COO.from_iter(x, shape=shape, fill_value=fill_value)
 
     raise NotImplementedError(
-        "Format not supported for conversion. Supplied type is "
-        "%s, see help(sparse.as_coo) for supported formats." % type(x)
+        f"Format not supported for conversion. Supplied type is "
+        f"{type(x)}, see help(sparse.as_coo) for supported formats."
     )
 
 
