@@ -1,11 +1,14 @@
-import numpy as np
-import sparse
-import pytest
 import operator
+
+import sparse
 from sparse import COO, DOK
 from sparse._compressed import GCXS
 from sparse._utils import assert_eq, random_value_array
 
+import pytest
+
+import numpy as np
+
 
 @pytest.mark.parametrize(
     "func",
@@ -89,9 +92,9 @@ def test_elemwise_inplace(func, format):
     ],
 )
 @pytest.mark.parametrize("format", [COO, GCXS, DOK])
-def test_elemwise_mixed(shape1, shape2, format):
+def test_elemwise_mixed(shape1, shape2, format, rng):
     s1 = sparse.random(shape1, density=0.5, format=format)
-    x2 = np.random.rand(*shape2)
+    x2 = rng.random(shape2)
 
     x1 = s1.todense()
 
@@ -99,9 +102,9 @@ def test_elemwise_mixed(shape1, shape2, format):
 
 
 @pytest.mark.parametrize("format", [COO, GCXS, DOK])
-def test_elemwise_mixed_empty(format):
+def test_elemwise_mixed_empty(format, rng):
     s1 = sparse.random((2, 0, 4), density=0.5, format=format)
-    x2 = np.random.rand(2, 0, 4)
+    x2 = rng.random((2, 0, 4))
 
     x1 = s1.todense()
 
@@ -123,10 +126,10 @@ def test_elemwise_unsupported(format):
 
 
 @pytest.mark.parametrize("format", [COO, GCXS, DOK])
-def test_elemwise_mixed_broadcast(format):
+def test_elemwise_mixed_broadcast(format, rng):
     s1 = sparse.random((2, 3, 4), density=0.5, format=format)
     s2 = sparse.random(4, density=0.5)
-    x3 = np.random.rand(3, 4)
+    x3 = rng.random((3, 4))
 
     x1 = s1.todense()
     x2 = s2.todense()
@@ -293,10 +296,7 @@ def test_trinary_broadcasting(shapes, func):
 @pytest.mark.parametrize("fraction", [0.25, 0.5, 0.75, 1.0])
 @pytest.mark.filterwarnings("ignore:invalid value")
 def test_trinary_broadcasting_pathological(shapes, func, value, fraction):
-    args = [
-        sparse.random(s, density=0.5, data_rvs=random_value_array(value, fraction))
-        for s in shapes
-    ]
+    args = [sparse.random(s, density=0.5, data_rvs=random_value_array(value, fraction)) for s in shapes]
     dense_args = [arg.todense() for arg in args]
 
     fs = sparse.elemwise(func, *args)
@@ -319,9 +319,7 @@ def test_sparse_broadcasting(monkeypatch):
             state["num_matches"] += 1
         return result
 
-    monkeypatch.setattr(
-        sparse._umath._Elemwise, "_get_func_coords_data", mock_unmatch_coo
-    )
+    monkeypatch.setattr(sparse._umath._Elemwise, "_get_func_coords_data", mock_unmatch_coo)
 
     xs * ys
 
@@ -343,9 +341,7 @@ def test_dense_broadcasting(monkeypatch):
             state["num_matches"] += 1
         return result
 
-    monkeypatch.setattr(
-        sparse._umath._Elemwise, "_get_func_coords_data", mock_unmatch_coo
-    )
+    monkeypatch.setattr(sparse._umath._Elemwise, "_get_func_coords_data", mock_unmatch_coo)
 
     xs + ys
 
@@ -372,9 +368,9 @@ def test_sparsearray_elemwise(format):
     assert_eq(fs, x + y)
 
 
-def test_ndarray_densification_fails():
+def test_ndarray_densification_fails(rng):
     xs = sparse.random((2, 3, 4), density=0.5)
-    y = np.random.rand(3, 4)
+    y = rng.random((3, 4))
 
     with pytest.raises(ValueError):
         xs + y
@@ -382,9 +378,10 @@ def test_ndarray_densification_fails():
 
 def test_elemwise_noargs():
     def func():
-        return np.float_(5.0)
+        return np.float64(5.0)
 
-    assert_eq(sparse.elemwise(func), func())
+    with pytest.raises(ValueError, match=r"None of the args is sparse:"):
+        sparse.elemwise(func)
 
 
 @pytest.mark.parametrize(
@@ -520,8 +517,8 @@ def test_scalar_output_nonzero_fv(func, scalar):
 def test_bitwise_binary(func, shape, format):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int_)
-    ys = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int64)
+    ys = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int64)
 
     x = xs.todense()
     y = ys.todense()
@@ -535,8 +532,8 @@ def test_bitwise_binary(func, shape, format):
 def test_bitwise_binary_inplace(func, shape, format):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int_)
-    ys = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int64)
+    ys = (sparse.random(shape, density=0.5, format=format) * 100).astype(np.int64)
 
     x = xs.todense()
     y = ys.todense()
@@ -552,11 +549,11 @@ def test_bitwise_binary_inplace(func, shape, format):
 def test_bitshift_binary(func, shape):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
 
     # Can't merge into test_bitwise_binary because left/right shifting
     # with something >= 64 isn't defined.
-    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int_)
+    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int64)
 
     x = xs.todense()
     y = ys.todense()
@@ -569,11 +566,11 @@ def test_bitshift_binary(func, shape):
 def test_bitshift_binary_inplace(func, shape):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
 
     # Can't merge into test_bitwise_binary because left/right shifting
     # with something >= 64 isn't defined.
-    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int_)
+    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int64)
 
     x = xs.todense()
     y = ys.todense()
@@ -586,11 +583,11 @@ def test_bitshift_binary_inplace(func, shape):
 
 @pytest.mark.parametrize("func", [operator.and_])
 @pytest.mark.parametrize("shape", [(2,), (2, 3), (2, 3, 4), (2, 3, 4, 5)])
-def test_bitwise_scalar(func, shape):
+def test_bitwise_scalar(func, shape, rng):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
-    y = np.random.randint(100)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
+    y = rng.integers(100)
 
     x = xs.todense()
 
@@ -600,14 +597,14 @@ def test_bitwise_scalar(func, shape):
 
 @pytest.mark.parametrize("func", [operator.lshift, operator.rshift])
 @pytest.mark.parametrize("shape", [(2,), (2, 3), (2, 3, 4), (2, 3, 4, 5)])
-def test_bitshift_scalar(func, shape):
+def test_bitshift_scalar(func, shape, rng):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
 
     # Can't merge into test_bitwise_binary because left/right shifting
     # with something >= 64 isn't defined.
-    y = np.random.randint(64)
+    y = rng.integers(64)
 
     x = xs.todense()
 
@@ -619,7 +616,7 @@ def test_bitshift_scalar(func, shape):
 def test_unary_bitwise_nonzero_output_fv(func, shape):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
     x = xs.todense()
 
     f = func(x)
@@ -633,11 +630,11 @@ def test_unary_bitwise_nonzero_output_fv(func, shape):
 
 @pytest.mark.parametrize("func", [operator.or_, operator.xor])
 @pytest.mark.parametrize("shape", [(2,), (2, 3), (2, 3, 4), (2, 3, 4, 5)])
-def test_binary_bitwise_nonzero_output_fv(func, shape):
+def test_binary_bitwise_nonzero_output_fv(func, shape, rng):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int_)
-    y = np.random.randint(1, 100)
+    xs = (sparse.random(shape, density=0.5) * 100).astype(np.int64)
+    y = rng.integers(1, 100)
 
     x = xs.todense()
 
@@ -655,9 +652,9 @@ def test_binary_bitwise_nonzero_output_fv(func, shape):
     [operator.mul, operator.add, operator.sub, operator.gt, operator.lt, operator.ne],
 )
 @pytest.mark.parametrize("shape", [(2,), (2, 3), (2, 3, 4), (2, 3, 4, 5)])
-def test_elemwise_nonzero_input_fv(func, shape):
-    xs = sparse.random(shape, density=0.5, fill_value=np.random.rand())
-    ys = sparse.random(shape, density=0.5, fill_value=np.random.rand())
+def test_elemwise_nonzero_input_fv(func, shape, rng):
+    xs = sparse.random(shape, density=0.5, fill_value=rng.random())
+    ys = sparse.random(shape, density=0.5, fill_value=rng.random())
 
     x = xs.todense()
     y = ys.todense()
@@ -667,11 +664,11 @@ def test_elemwise_nonzero_input_fv(func, shape):
 
 @pytest.mark.parametrize("func", [operator.lshift, operator.rshift])
 @pytest.mark.parametrize("shape", [(2,), (2, 3), (2, 3, 4), (2, 3, 4, 5)])
-def test_binary_bitshift_densification_fails(func, shape):
+def test_binary_bitshift_densification_fails(func, shape, rng):
     # Small arrays need high density to have nnz entries
     # Casting floats to int will result in all zeros, hence the * 100
-    x = np.random.randint(1, 100)
-    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int_)
+    x = rng.integers(1, 100)
+    ys = (sparse.random(shape, density=0.5) * 64).astype(np.int64)
 
     y = ys.todense()
 
@@ -712,3 +709,18 @@ def test_nanmean_regression(dtype):
     array = np.array([0.0 + 0.0j, 0.0 + np.nan * 1j], dtype=dtype)
     sparray = sparse.COO.from_numpy(array)
     assert_eq(array, sparray)
+
+
+# Regression test for gh-580
+@pytest.mark.filterwarnings("error")
+def test_no_deprecation_warning():
+    a = np.array([1, 2])
+    s = sparse.COO(a, a, shape=(3,))
+    assert_eq(s == s, np.broadcast_to(True, s.shape))
+
+
+# Regression test for gh-587
+def test_no_out_upcast():
+    a = sparse.COO([[0, 1], [0, 1]], [1, 1], shape=(2, 2))
+    with pytest.raises(TypeError):
+        a *= 0.5
